// A simple program to visualize that diﬀerent class breaks yield diﬀerent choropleth maps - written by Michael Regenscheit (September, 2009)
// Note: the data file (polygons) must be in the same directory

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Polygon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

public class ChoroplethMap extends JFrame {

	private static final long serialVersionUID = 3153442147800045018L;

	// the visualization panel
	private MapPanel m_mapPanel;

	// Create the UI and its components
	ChoroplethMap(String polygonsFile) {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));

		m_mapPanel = new MapPanel(polygonsFile);
		m_mapPanel.addMouseMotionListener(new MouseMotionListener() {

			public void mouseDragged(MouseEvent e) {}

			public void mouseMoved(MouseEvent e) {
				m_mapPanel.showValue(e);
			}
			
		});

		ButtonGroup buttonGroup = new ButtonGroup();
		JRadioButton rb1 = new JRadioButton();
		buttonGroup.add(rb1);
		JRadioButton rb2 = new JRadioButton();
		buttonGroup.add(rb2);
		JRadioButton rb3 = new JRadioButton();
		buttonGroup.add(rb3);
		JRadioButton rb4 = new JRadioButton();
		buttonGroup.add(rb4);
		JRadioButton rb5 = new JRadioButton();
		buttonGroup.add(rb5);
		rb1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				m_mapPanel.setChoosen(0);
				m_mapPanel.repaint();
			}
			
		});
		rb2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				m_mapPanel.setChoosen(1);
				m_mapPanel.repaint();
			}
			
		});
		rb3.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				m_mapPanel.setChoosen(2);
				m_mapPanel.repaint();
			}
			
		});
		rb4.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				m_mapPanel.setChoosen(3);
				m_mapPanel.repaint();
			}
			
		});
		rb5.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				m_mapPanel.setChoosen(4);
				m_mapPanel.repaint();
			}
			
		});
		rb1.setSelected(true);

		Box radioButtonBox = Box.createHorizontalBox();
		radioButtonBox.add(new JLabel("Different classes:"));
		radioButtonBox.add(rb1);
		radioButtonBox.add(rb2);
		radioButtonBox.add(rb3);
		radioButtonBox.add(rb4);
		radioButtonBox.add(rb5);
		
		panel.add(m_mapPanel);
		panel.add(radioButtonBox);
		panel.add(Box.createVerticalStrut(25));

		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.add(panel);
		this.setTitle("Choropleth Map - Population Density");
		this.setResizable(false);
		this.pack();
		this.setVisible(true);
	}

	// Start it up
	public static void main(String[] agrs) {
		new ChoroplethMap("polygons");
	}

	class MapPanel extends JPanel {

		private static final long serialVersionUID = -7212812245307971947L;

		ArrayList<Polygon> m_polygons;
		ArrayList<Integer> m_polygonsValue;
		Color[] m_colors = { new Color(255, 255, 178), new Color(254, 204, 92),
				new Color(253, 141, 60), new Color(227, 26, 28) };

		int[] m_alternative1 = { 200, 400, 600 };
		int[] m_alternative2 = { 100, 550, 1000 };
		int[] m_alternative3 = { 100, 300, 600 };
		int[] m_alternative4 = { 175, 350, 525 };
		int[] m_alternative5 = { 150, 300, 600 };
		
		int m_choosen = 0;

		// starts the polygon file loader
		MapPanel(String polygonsFile) {
			loadPolygons(polygonsFile);
		}

		public void paint(Graphics g) {
			super.paintComponent(g); // clears background
			
			// set the class breaks
			int[] borders;
			if(m_choosen == 0) {
				borders = m_alternative1;
			} else if(m_choosen == 1) {
				borders = m_alternative2;
			}else if(m_choosen == 2) {
				borders = m_alternative3;
			}else if(m_choosen == 3) {
				borders = m_alternative4;
			} else {
				borders = m_alternative5;
			}
			
			// paint the polygons
			for ( int i = 0; i< m_polygons.size(); i++) {
				if (m_polygonsValue.get(i)<borders[0]) {
					g.setColor(m_colors[0]);
				} else if (m_polygonsValue.get(i)<borders[1]) {
					g.setColor(m_colors[1]);
				}else if (m_polygonsValue.get(i)<borders[2]) {
					g.setColor(m_colors[2]);
				} else {
					g.setColor(m_colors[3]);
				}
				g.fillPolygon(m_polygons.get(i));
				g.setColor(Color.BLACK);
				g.drawPolygon(m_polygons.get(i));
			}
			
			// paint the class breaks
			for (int i = 0; i<m_colors.length; i++) {
				g.setColor(m_colors[i]);
				g.fillRect(0 +(100*i), 350, 100 +(100*i), 25);
				g.setColor(Color.BLACK);
				g.drawRect(0 +(100*i), 350, 100 +(100*i), 25);
				if (i!= m_colors.length-1) {
				g.drawString("< " +borders[i], 30+(100*i), 368);
				} else {
					g.drawString(">= " +borders[i-1], 25+(100*i), 368);
				}
			}
		}

		public Dimension getPreferredSize() {
			return new Dimension(400, 400);

		}
		
		public void setChoosen(int choosen) {
			m_choosen = choosen;
		}

		// loads the polygons
		private void loadPolygons(String polygonsFile) {
			m_polygons = new ArrayList<Polygon>();
			m_polygonsValue = new ArrayList<Integer>();
			FileReader reader;
			try {
				reader = new FileReader(polygonsFile);
				BufferedReader buffer = new BufferedReader(reader);
				String line;
				Polygon p = new Polygon();
				while ((line = buffer.readLine()) != null) {
					StringTokenizer toker = new StringTokenizer(line, ",");
					if (toker.countTokens() == 0) {
						// do nothing
					} else if (toker.countTokens() == 1) {
						m_polygonsValue
								.add(Integer.parseInt(toker.nextToken()));
						m_polygons.add(p);
						p = new Polygon();
					} else {
						p.addPoint(Integer.parseInt(toker.nextToken()), Integer
								.parseInt(toker.nextToken()));
					}
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		// displays the population density (value) of a polygon as a ToolTipText
		public void showValue(MouseEvent e) {
			boolean found = false;
			for(int i = 0; i<m_polygons.size(); i++) {
				if(m_polygons.get(i).contains(e.getX(), e.getY())) {
					this.setToolTipText("Population Density: "+m_polygonsValue.get(i));
					found = true;
					return;
				}
			}
			if(!found) this.setToolTipText("");
		}

	}
}
